/****************************************************************************
*	Title	:	mTouch 16Button Module										*
*	UpDate	:	2008.5.08													*
*																			*
*	Device	:	PIC16F727-I/PT												*
*	OSC		:	4Mhz( Internal OSC 4Mhz )									*
*																			*
*	Compiler:	Hi-tech	V9.60 std											*
*	IDE		:	MPLAB IDE V8.10												*
*	Hardware:	Captouch Black Demo Board									*
*****************************************************************************

****************************************************************************/
#include	<pic.h>
#include	"ssp.h"
#include	"commdef.h"
//***************************************************************************
__CONFIG(INTIO & WDTDIS & PWRTDIS & MCLREN & BORDIS & UNPROTECT & PLLEN);
__CONFIG(VCAPRA0);
//***************************************************************************


#define TestPort1		RC7
#define TestPort1_Tris	TRISC7
#define TestPort2		RA3
#define TestPort2_Tris	TRISA3

//----- Port Define -----//
#define PIC16F727_IRQ	RC6


//-----I2C Port Define -----//
#define SCL			RC3		
#define SCL_DIR		TRISC3
#define SDA			RC4
#define SDA_DIR		TRISC4

//-----Initial_I2C_Slave Routine -----//
void Initial_I2C_Slave(void);
void ISR_I2CSlave(void);
void Initial_UART(void);

#define	INPUT		1
#define	OUTPUT		0

#define	DIR_DRV1 	TRISA3
#define	DIR_DRV2	TRISA2
#define	DIR_DRV3	TRISA1
#define	DIR_DRV4 	TRISE0

#define	DRV1 		RA3
#define	DRV2		RA2
#define	DRV3		RA1 
#define	DRV4		RE0

#define	DIR_DRVS1	TRISE1
#define	DIR_DRVS2	TRISE2
#define	DIR_DRVS3 	TRISA7
#define	DIR_DRVS4	TRISA6
#define	DIR_DRVS5	TRISC0
#define	DIR_DRVPWR	TRISC1


#define	DRVS1		RE1
#define DRVS2		RE2
#define	DRVS3		RA7
#define	DRVS4		RA6
#define	DRVS5		RC0
#define	DRVPWR		RC1

#define SLIDER_ADDR	0b00010000
#define MULTI_ADDR	0b00100000
#define WHEEL_ADDR	0b00110000	

#define TMR0VALUE		14			// TMR0 Interrupt Time 1msec @ 4Mhz, 1:4 Prescaler

#define THRESHOLD_50	1			// avg >> 1 = 1/2   = 50%   as a threshold value below average
#define THRESHOLD_25	2			// avg >> 2 = 1/4   = 25%   as a threshold value below average
#define THRESHOLD_12P5	3			// avg >> 3 = 1/8   = 12.5% as a threshold value below average
#define THRESHOLD_6P25	4			// avg >> 4 = 1/16  = 6.25% as a threshold value below average
#define THRESHOLD_3P12	5			// avg >> 5 = 1/32  = 3.12% as a threshold value below average
#define THRESHOLD_1P56	6			// avg >> 6 = 1/64  = 1.56% as a threshold value below average
#define THRESHOLD_0P78	7			// avg >> 7 = 1/128 = 0.78% as a threshold value below average

#define HYSTERESIS		3

/** Variables **/

bank1 volatile unsigned int		AverageValue[NUM_BTTNS] ;
bank1 volatile unsigned int		MakeValue[NUM_BTTNS] ;
bank1 volatile unsigned char	ChannelMake[NUM_BTTNS] ;

bank2 volatile unsigned int		RAW[NUM_BTTNS] ;
bank2 volatile unsigned int		AverageAcc[NUM_BTTNS] ;
bank2 volatile unsigned char	SampleTime[NUM_BTTNS] ;

bank3 volatile unsigned int	 	GUARD[NUM_BTTNS];
bank3 volatile unsigned int  	TRIP[NUM_BTTNS] ;
bank3 volatile unsigned char 	Threshold[NUM_BTTNS];			// Threshold for each button

typedef struct{
	char BTNPWR :1;						// {byte 0}
	char BTNS5  :1;
	char BTNS4  :1;
	char BTNS3  :1;
	char BTNS2  :1;
	char BTNS1  :1;
	char BTN0 	:1;
	char BTN9  	:1;
	char BTN8  	:1;
	char BTN7  	:1;
	char BTN6  	:1;
	char BTN5 	:1;
	char BTN4 	:1;
	char BTN3 	:1;
	char BTN2 	:1;
	char BTN1 	:1;
}BButtons;
volatile BButtons Buttons;

volatile unsigned char			Index,Counter=0;
volatile unsigned int 			ButtonData, ButtonDataBuff;
volatile unsigned int			ButtonStatus ;
volatile unsigned int			OldButtonStatus = 0 ;

volatile unsigned char			ScanTimeBase=0 ;


/** Prototypes **/
void Init(void);
void RestartTimers(void);
void SetNextChannel(void);
void interrupt isr(void);
void SLEEP_NOP(void);
void SaveButtonData(void);
void LED_Actions(void) ;

void	Send_Int2UART(int);
void	Send_Average2UART(void);
void	ReduceAverageValue(void) ;
//**************************************************************************
void Delay_Routine(unsigned int Delay)
{
	while(--Delay){CLRWDT();}
}
//******************************************************************************************
void main(void)
{
	unsigned char	ClearLoop ;

	for ( ClearLoop = 0 ; ClearLoop < NUM_BTTNS ; ClearLoop++)
	{
		ChannelMake[ClearLoop] = 0 ;
		SampleTime[ClearLoop] = 0 ;
		AverageValue[ClearLoop] = 0 ;
	}
	
	Init();
	Initial_UART() ;

	DIR_DRV1 = 	INPUT ;
	DIR_DRV2 = 	INPUT ;
	DIR_DRV3 = 	INPUT ;
	DIR_DRV4 = 	INPUT ;
	DIR_DRVS1 = OUTPUT ;
	DIR_DRVS2 = OUTPUT ;
	DIR_DRVS3 = OUTPUT ;
	DIR_DRVS4 = OUTPUT ;
	DIR_DRVS5 = OUTPUT ;
	DIR_DRVPWR = OUTPUT ;


	while(1)
	{

		//		Send_Average2UART() ;	// 	Ȯɥ Disable UART ǰe

	 if ( ScanTimeBase > 5 )
	 {
		ScanTimeBase = 0 ;

		// 		LED_Actions() ;

	 }  //		End of ScanTimeBase statement 
	}
}


//******************************************************************************************
void Init(void) 
	 {
	OSCCON = 0b00110000;				// Internal 16Mhz @PLL On

	// Port Description
	//--------------------------------------------------------------------------
	//			7		6		5		4		3		2		1		0       |
	// PORTA	.       	    CPS7(0) CPS6(*) Test2							|    
	// PORTB    .       .       CPS5(A) CPS4(3) CPS3(2) CPS2(1) CPS1(B) CPS0(6) |
	// PORTC	Test1   IRQ     MISO    SDA     SCL     .       .       .       | 
	// PORTD	CPS15(5)CPS14(4)CPS13(C)CPS12(9)CPS11(8)CPS10(7)CPS9(D) CPS8(#) | 
	//--------------------------------------------------------------------------
	TRISA = 0b00110001;
	ANSELA= 0b00110001;

	TRISB = 0b00111111;
	ANSELB= 0b00111111;

	TRISC = 0b00111111;

	TRISD = 0b11111111;
	ANSELD= 0b11111111;

	ANSELE = 0b11111000 ;				// RE0.. RE2 = Digital mode

	// Initialize for Timer0 time base
	OPTION = 0b11000011;				// Timer0 init ( Prescaler:TMR0, 1:16 )
	T0IF = 0;							// enable tmr0 intpt
	T0IE = 1;	

	// Initialize for Timer1 time Resource
	T1CON  = 0b11000101;				// Timer1 enable, system clock, 1:1 prescale, 
		
	// Cap Sense Module
	CPSCON0 = 0b10001110;				// control settings
	CPSCON1 = 0x00;						// init to channel select = 0 (4 LSb's)
										// Start program ready to scan first channel
	
	SSPInit() ;

	for (Index=0; Index<NUM_BTTNS; Index++) 
	{
		ChannelMake[Index] = 0 ;
		SampleTime[Index] = 0 ;
		AverageValue[Index] = 0 ;
		AverageAcc[Index] = 0 ;
		RAW[Index] = 0 ;
		GUARD[Index] = 0 ;
		TRIP[Index] = 0;
	}

		ButtonStatus = 0 ;
		ButtonDataBuff = 0 ;

	Index = 0x00 ;
	// Interrupt Enable
	GIE = 1;
 }

void	Initial_UART(void)
{
		//	Initialize the UART @ 16Mhz , 19200 , N ,8 , 1 

		TRISC6 = 1 ;
		TRISC7 = 1 ;
		SPBRG  = 12 ;
		TXSTA  = 0x24 ;
		RCSTA  = 0x80 ;
}
//******************************************************************************************
void interrupt isr(void) 
  {

	//----- I2C Slave Interrupt -----//
	if(SSPIE==1 && SSPIF==1)
	{
		SSPISR();
	}

	//----- TMR0 Interrupt -----//
	if (T0IE && T0IF)
		{
		//	Remark by Calvin ... start
		//  TestPort1^=1;
		//	Remark by Calvin ... end
		ScanTimeBase++ ;

		TMR0=TMR0VALUE;
		T0IF = 0;						// Clear T0IF every time
		
		TMR1ON = 0;						// Timer1 off
					
		if ( SampleTime[Index] < 4 )
			{
				AverageAcc[Index] += TMR1L + (unsigned int)(TMR1H << 8);
				RAW[Index] = TMR1L + (unsigned int)(TMR1H << 8);
				SampleTime[Index]++ ;
			}
		if ( SampleTime[Index] == 4 )
			{
						AverageValue[Index] = (AverageAcc[Index] >> 2);
						RAW[Index] = AverageValue[Index] ;
						AverageAcc[Index] = 0 ;
						SampleTime[Index] = 0 ;
			}

		// Threshold Define for each Button
	if ( (ChannelMake[Index] == 0)&&(AverageValue[Index]>200))
	{

		switch (Index){
			case 0:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 1:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 2:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 3:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 4:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 5:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 6:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 7:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 8:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 9:		Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+3;  break;
			case 10:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+2;  break;
			case 11:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+2;  break;
			case 12:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+2;  break;
			case 13:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+2;  break;
			case 14:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_1P56)+2;  break;
			case 15:	Threshold[Index]  = (AverageValue[Index]>>THRESHOLD_3P12)+3;  break;
			default:	break;
		}

		// Button Pressed (no hysteresis provided)

		if ((RAW[Index] < (AverageValue[Index] - Threshold[Index])))
			{
				ReduceAverageValue( ) ;
				switch(Index){
				case 0:		
							Buttons.BTN1  = 1;  
							ChannelMake[0] = 1;
							MakeValue[0] = RAW[Index];
							AverageValue[0] = RAW[Index] ;
							break;										// The current button is pressed   
				case 1:													// Relay button pressed information to application
							Buttons.BTN2  = 1; 
							ChannelMake[1] = 1;
							MakeValue[1] = RAW[Index];
							AverageValue[1] = RAW[Index] ;
							break;					
				case 2:		Buttons.BTN3  = 1;  
							ChannelMake[2] = 1;
							MakeValue[2] = RAW[Index]-2;
    						AverageValue[2] = RAW[Index] ;
							break;	
				case 3:		Buttons.BTN4  = 1;	
							ChannelMake[3] = 1;
							MakeValue[3] = RAW[Index]-2;
							AverageValue[3] = RAW[Index] ;
							break;  
				case 4:		Buttons.BTN5  = 1;  
							ChannelMake[4] = 1;
							MakeValue[4] = RAW[Index]-2;
							AverageValue[4] = RAW[Index] ;
							break;	
				case 5:		Buttons.BTN6  = 1;  
							ChannelMake[5] = 1;
							MakeValue[5] = RAW[Index]-2;
							AverageValue[5] = RAW[Index] ;
							break;
				case 6:		Buttons.BTN7  = 1;  
							ChannelMake[6] = 1;
							MakeValue[6] = RAW[Index]-2;
							AverageValue[6] = RAW[Index] ;
							break;
				case 7:		Buttons.BTN8  = 1;  
							ChannelMake[7] = 1;
							MakeValue[7] = RAW[Index]-2;
							AverageValue[7] = RAW[Index] ;
							break;
				case 8:		Buttons.BTN9  = 1;  
							ChannelMake[8] = 1;
							MakeValue[8] = RAW[Index]-2;
							AverageValue[8] = RAW[Index] ;
							break;
				case 9:		Buttons.BTN0  = 1;  
							ChannelMake[9] = 1;
							MakeValue[9] = RAW[Index]-2;
							AverageValue[9] = RAW[Index] ;
							break;
				case 10:	Buttons.BTNS1 = 1;  
							ChannelMake[10] = 1;
							MakeValue[10] = RAW[Index];
							AverageValue[10] = RAW[Index] ;
							break;
				case 11:	Buttons.BTNS2 = 1;  
							ChannelMake[11] = 1;
							MakeValue[11] = RAW[Index];
							AverageValue[11] = RAW[Index] ;
							break;
				case 12:	Buttons.BTNS3 = 1;  
							ChannelMake[12] = 1;
							MakeValue[12] = RAW[Index];
							AverageValue[12] = RAW[Index] ;
							break;
				case 13:	Buttons.BTNS4 = 1;  
							ChannelMake[13] = 1;
							MakeValue[13] = RAW[Index];
							AverageValue[13] = RAW[Index] ;
							break;
				case 14:	Buttons.BTNS5 = 1;  
							ChannelMake[14] = 1;
							MakeValue[14] = RAW[Index];
							AverageValue[14] = RAW[Index] ;
							break;
				case 15:	Buttons.BTNPWR= 1;  
							ChannelMake[15] = 1;
							MakeValue[15] = RAW[Index];
							AverageValue[15] = RAW[Index] ;
							break;
				default:	break;
			}
		}

	}
		// Button unpressed (hysteresis provided)
		if ( ChannelMake[Index] && (AverageValue[Index] > 100))	
		{
//			if(RAW[Index] > MakeValue[Index]+3)
//			if ( AverageValue[Index] > ( MakeValue[Index]-10 ))
			if (RAW[Index] > (AverageValue[Index] + 3))
			{
			switch(Index){
				case 0:		
							Buttons.BTN1  = 0;  
							ChannelMake[0] = 0 ;
							break;						// The current button is unpressed    
				case 1:		Buttons.BTN2  = 0;  		// Relay button pressed information to application
							ChannelMake[1] = 0 ;						
							break;						
				case 2:		Buttons.BTN3  = 0;  
							ChannelMake[2] = 0 ;
							break;		
				case 3:		Buttons.BTN4  = 0;	
							ChannelMake[3] = 0 ;
							break;    
				case 4:		Buttons.BTN5  = 0;
							ChannelMake[4] = 0 ;  
							break;		
				case 5:		Buttons.BTN6  = 0;  
							ChannelMake[5] = 0 ;
							break;  
				case 6:		Buttons.BTN7  = 0;  
							ChannelMake[6] = 0 ;
							break;  	
				case 7:		Buttons.BTN8  = 0;  
							ChannelMake[7] = 0 ;
							break;    
				case 8:		Buttons.BTN9  = 0;  
							ChannelMake[8] = 0 ;
							break;  	
				case 9:		Buttons.BTN0  = 0;  
							ChannelMake[9] = 0 ;
							break;  
				case 10:	Buttons.BTNS1 = 0;  
							ChannelMake[10] = 0 ;
							break;  	
				case 11:	Buttons.BTNS2 = 0;  
							ChannelMake[11] = 0 ;
							break;    
				case 12:	Buttons.BTNS3 = 0;  
							ChannelMake[12] = 0 ;
							break;  	
				case 13:	Buttons.BTNS4 = 0;  
							ChannelMake[13] = 0 ;
							break;  
				case 14:	Buttons.BTNS5 = 0;  
							ChannelMake[14] = 0 ;
							break;  	
				case 15:	Buttons.BTNPWR= 0;  
							ChannelMake[15] = 0 ;
							break;    
				default:	break;
			}   
	
		}
	}
		LED_Actions() ;

		SetNextChannel();				// Set up for next channel
		RestartTimers();				// Restart timer1
		SaveButtonData();				// Save Buttone Data
	}
}

//		************************************************************************
//		**	Cp changed when Anykey pressd 
//		**	Set AverageValue & SampleTime in order to re-calculate AverageValue
//		**  purpose : to avoid the incorrect judgement when cp changed 
//		************************************************************************
void	ReduceAverageValue(void)
{
		unsigned char	ReduceLoop;
		for (ReduceLoop = 0 ; ReduceLoop <= 15 ; ReduceLoop++)
			{
				AverageValue[ReduceLoop] = 10 ;
				AverageAcc[ReduceLoop] = 0 ;
				SampleTime[ReduceLoop] = 0 ;
			}
}
//******************************************************************************************
void SetNextChannel(void) 
{
	if(++Index>=NUM_BTTNS) 			Index=0;
		CPSCON1 = Index;				// Select external pin CPS0..CPS15
}
//******************************************************************************************
void RestartTimers(void) 
{
	TMR1L	= 0;						// reset Timer1
	TMR1H	= 0;						//   "     "
	TMR1ON	= 1;						// restart Timer1
	TMR0	= TMR0VALUE;				// reset Timer0
	T0IF	= 0;						// clear the interrupt
}
//******************************************************************************************
void SaveButtonData(void)
{
	unsigned char i,j;

	ButtonDataBuff=0;
	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN1;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN2;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN3;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN4;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN5;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN6;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN7;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN8;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN9;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN0;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS1;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS2;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS3;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS4;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS5;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNPWR;	
	
	if(ButtonData!=ButtonDataBuff){
		if(ButtonData>ButtonDataBuff){
			ButtonData=ButtonDataBuff=0;
		}	
		ButtonData=ButtonDataBuff;
	}
}


void	LED_Actions(void)
{

	ButtonDataBuff=0;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNPWR;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS5;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS4;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS3;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS2;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTNS1;
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN0;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN9;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN8;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN7;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN6;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN5;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN4;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN3;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN2;	
	ButtonDataBuff=(ButtonDataBuff<<1)|Buttons.BTN1;	


		ButtonStatus = ButtonDataBuff ;
		if (OldButtonStatus == ButtonStatus )	return ;
			OldButtonStatus = ButtonStatus ;

//		return ;

//	***	ON/OFF control for LED0 .. LED3
		if ( Buttons.BTN1 == 1 )
			{
				DRV1 = 1 ;
				DRV2 = 0 ;
				DIR_DRV3 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV2 = OUTPUT ;
			}

		else if ( Buttons.BTN2 == 1 )
			{
				DRV1 = 1 ;
				DRV4 = 0 ;
				DIR_DRV2 = INPUT ;
				DIR_DRV3 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV4 = OUTPUT ;
			}

		else if ( Buttons.BTN3 == 1 )
			{
				DRV1 = 1 ;
				DRV3 = 0 ;
				DIR_DRV2 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV3 = OUTPUT ;
			}
		else if ( Buttons.BTN4 == 1 )
			{
				DRV1 = 0 ;
				DRV2 = 1 ;
				DIR_DRV3 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV2 = OUTPUT ;
			}

		else if ( Buttons.BTN5 == 1 )
			{
				DRV1 = 0 ;
				DRV4 = 1 ;
				DIR_DRV2 = INPUT ;
				DIR_DRV3 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV4 = OUTPUT ;
			}

		else if ( Buttons.BTN6 == 1 )
			{	
				DRV1 = 0 ;
				DRV3 = 1 ;
				DIR_DRV2 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV1 = OUTPUT ;
				DIR_DRV3 = OUTPUT ;				
			}
		else if ( Buttons.BTN7 == 1 )
			{	
				DRV2 = 1 ;
				DRV3 = 0 ;
				DIR_DRV1 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV2 = OUTPUT ;
				DIR_DRV3 = OUTPUT ;				
			}
		else if ( Buttons.BTN8 == 1 )
			{	
				DRV2 = 0 ;
				DRV3 = 1 ;
				DIR_DRV1 = INPUT ;
				DIR_DRV4 = INPUT ;
				DIR_DRV2 = OUTPUT ;
				DIR_DRV3 = OUTPUT ;				
			}
		else if ( Buttons.BTN9 == 1 )
			{	
				DRV3 = 0 ;
				DRV4 = 1 ;
				DIR_DRV1 = INPUT ;
				DIR_DRV2 = INPUT ;
				DIR_DRV3 = OUTPUT ;
				DIR_DRV4 = OUTPUT ;				
			}
		else if ( Buttons.BTN0 == 1 )
			{	
				DRV3 = 1 ;
				DRV4 = 0 ;
				DIR_DRV1 = INPUT ;
				DIR_DRV2 = INPUT ;
				DIR_DRV3 = OUTPUT ;
				DIR_DRV4 = OUTPUT ;				
			}
		else 
			{
				DIR_DRV1 = INPUT ;
				DIR_DRV2 = INPUT ;
				DIR_DRV3 = INPUT ;	
				DIR_DRV4 = INPUT ;	
			}


		if ( Buttons.BTNS1 == 1 )		DRVS1 = 1 ;
		else							DRVS1 = 0 ;

		if ( Buttons.BTNS2 == 1 )		DRVS2 = 1 ;
		else							DRVS2 = 0 ;
				
		if ( Buttons.BTNS3 == 1 ) 		DRVS3 = 1 ;
		else							DRVS3 = 0 ;

		if ( Buttons.BTNS4 == 1 )		DRVS4 = 1 ;
		else							DRVS4 = 0 ;

		if ( Buttons.BTNS5 == 1 )		DRVS5 = 1 ;
		else							DRVS5 = 0 ;

		if ( Buttons.BTNPWR == 1 )		DRVPWR = 1 ;
		else							DRVPWR = 0 ;
}

void	Send_Average2UART(void)
{
	unsigned char	AvgLoop ;

		while ( TXIF == 0 );
			TXREG = 0xf0 ;
			asm("nop");
		while ( TXIF == 0 ) ;
			TXREG = 0xaa ;
			asm("nop");
		while ( TXIF == 0 ) ;
			TXREG = 0xf1 ;
			asm("nop");
		while ( TXIF == 0) ;
			TXREG = 0xbb ;
			asm("nop");
		for ( AvgLoop = 0 ; AvgLoop < 16 ; AvgLoop ++) 	
		{
			Send_Int2UART(AverageValue[AvgLoop]) ;
		}
			Send_Int2UART(ButtonStatus) ;
}

void	Send_Int2UART(int	IntValue)
{
		while( TXIF == 0 ) ;
			TXREG = (char)(IntValue & 0x00ff) ;
			asm("nop") ;
		while( TXIF == 0 ) ;
			TXREG = (char)((IntValue >> 8) & 0xff) ;
			asm("nop") ;

}
